import os

from celery import Celery
from kombu import Exchange, Queue

from hijiki.broker.broker_data import get_broker_url, init_os_environ


class HijikiBroker:
    celery_broker = None

    def bind_queues(self, queues_names):
        for name in queues_names:
            dlx = Exchange(f'dlq_{name}_exchange', type='topic')

            dead_letter_queue = Queue(f'dlq_{name}', dlx, routing_key='*', queue_arguments={'x-queue-type': 'quorum'})
            dead_letter_queue.bind(self.celery_broker.broker_connection()).declare()

    def __init__(self, app_name, host, username, password, port):
        init_os_environ(host, username, password, port)
        self.celery_broker = Celery(app_name, broker=get_broker_url(), set_as_current=True)


    def get_celery_broker(self):
        return self.celery_broker

    def bind_queues(self, queues_names):
        self.bind_queues(queues_names)


import threading
import time
import unittest

from testcontainers.core.container import DockerContainer

from hijiki.broker.hijiki_rabbit import HijikiQueueExchange, HijikiRabbit
from hijiki.publisher.Publisher import Publisher

result_event_list = []


class Runner():
    qs = [HijikiQueueExchange('teste1', 'teste1_event'), HijikiQueueExchange('teste2', 'teste2_event')]
    gr = HijikiRabbit().with_queues_exchange(qs) \
        .with_username("user") \
        .with_password("pwd") \
        .with_host("localhost") \
        .with_port(5672) \
        .build()

    threads = []

    @gr.task(queue_name="teste1")
    def internal_consumer(self):
        print("consumiu")
        result_event_list.append('recebeu evento')

    def run(self):
        t = threading.Thread(target=self.gr.run)
        self.threads.append(t)
        t.start()

    def stop(self):
        self.gr.terminate()

    def __del__(self):
        super()


class TestPublisherConsumer(unittest.TestCase):
    runner = None

    def setUp(self):
        self.runner = Runner()
        self.runner.run()
        self.pub = Publisher("localhost", "user", "pwd", 5672)

    def tearDown(self):
        self.runner.stop()

    def test_publish_a_message(self):
        self.pub.publish_message('teste1_event', '{"value": "Esta é a mensagem"}')

    def test_consume_a_message(self):
        self.pub = Publisher("localhost", "user", "pwd", 5672)
        time.sleep(5)
        self.pub.publish_message('teste1_event', '{"value": "Esta é a mensagem"}')
        time.sleep(1)
        self.assertEqual(len(result_event_list), 1)
